/* krubik.cpp
 
  This is my third application using Qt/Kde, it's a 3D model of the
  Rubik's Cube.

  This program is based on original code from : Karl Hornell,
  see http://www.tdb.uu.se/~karl/java/rubik.html

  Copyright (C) 1997	Pieter Eendebak	(pte@ddsw.nl)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

#include "includes.h"

#include "defines.h"
#include "cube.h"
#include "dialog.h"

#include "krubik.moc"

KApplication *krubik;


/* ##########################
 * #	class KRubik                 #
 * ##########################
 */

void KRubik::createMenu()
{
	int id ;

	file = new QPopupMenu();
	id = file->insertItem(klocale->translate("&New cube"),
				this, SLOT(newCube()) );
	id = file->insertItem(klocale->translate("Save cube"),
				this, SLOT(saveCube()) );
	id = file->insertItem(klocale->translate("Load cube"),
				this, SLOT(loadCube()) );
	
	file->insertSeparator(-1);
	id = file->insertItem(klocale->translate("&Quit"), this, SLOT(quitapp()) );
	
	options = new QPopupMenu();
	id = options->insertItem(	klocale->translate("C&onfigure colors"),
				this,
				SLOT(configColors()) );
	id = options->insertItem(	klocale->translate("&Reset colors"),
				this,
				SLOT(resetColors()) );

	id = options->insertItem(	klocale->translate("&Configure keys"),
				 this,
				 SLOT(configkeys()) );

	if ( showstatus )
	{
		statusID= options->insertItem(
				klocale->translate("Hide Status Bar"),
				this, SLOT(toggleStatusBar()) );
	}
	else
	{
		statusID= options->insertItem(
				klocale->translate("Show Status Bar"),
				this, SLOT(toggleStatusBar()) );
	}

	help = new QPopupMenu();
	id = help->insertItem(klocale->translate("&Help"), this, SLOT(helpmenu()) );
	id = help->insertItem(klocale->translate("&About..."), this, SLOT(about()) );
	
	menu = new KMenuBar(this, "menu");
	id = menu->insertItem(klocale->translate("&File"), file, 5);
	id = menu->insertItem(klocale->translate("&Options"), options);
	menu->insertSeparator(-1);
	id = menu->insertItem(klocale->translate("&Help"), help);

	setMenu(menu);
}

void KRubik::resizeEvent( QResizeEvent * )
{
	resizeEvent();
}

void KRubik::resizeEvent()
{

	QSize s = size();

	int hoog = s.height();
	int breed = s.width();

	int sbar=0;
	if ( showstatus ) sbar=20;

	int bbreed = breed - 140;
	int bhoog = hoog - 60 -sbar;

	menu->setGeometry(0,0,breed,30);
	box->setGeometry(10,40,bbreed, bhoog);
	kubus->setGeometry(25,25, bbreed-50, bhoog-50);

	moves->setGeometry(breed-110,50,100,30);
	counter->setGeometry(breed-110,80,100,30);

	clean->setGeometry(breed-110,120,100,30);
	scramble->setGeometry(breed-110,170,100,30);

	statusbar->setGeometry(0, hoog-20,breed, sbar);
}

void KRubik::createStatusBar()
{
	statusbar = new KStatusBar(this);

	statusbar->insertItem(
		klocale->translate("Moves : 0000 "),
		STATUS_MOVES);
	statusbar->insertItem(
		klocale->translate("Time : 0000 "),
		STATUS_TIME);
	statusbar->insertItem(
		klocale->translate("Wrong pieces : 00 "),
		STATUS_SCAT);
	statusbar->insertItem("", STATUS_FAKE);

	setStatusBar ( statusbar );

	if ( showstatus )	enableStatusBar ( KStatusBar::Show );
	else			enableStatusBar ( KStatusBar::Hide );

	status();
}

void KRubik::toggleStatusBar()
{
	if ( showstatus )
	{
		showstatus=false;
		enableStatusBar( KStatusBar::Hide );
		options->changeItem(
				klocale->translate("Show Status Bar"),
				statusID );
	}
	else
	{
		showstatus=true;
		enableStatusBar( KStatusBar::Show );
		options->changeItem(
				klocale->translate("Hide Status Bar"),
				statusID );
	}

	resizeEvent();
}

void KRubik::status()
{
	QString tmp;

	tmp.sprintf("%s : %4d ",
		klocale->translate("Moves"),
		kubus->getMoves()  ); 
	statusbar->changeItem( tmp.data(), STATUS_MOVES);

	tmp.sprintf("%s : %2d ", klocale->translate("Wrong pieces"),
				kubus->scattered() );
	statusbar->changeItem( tmp.data(), STATUS_SCAT );

	tmp.sprintf("%s : %4d ", klocale->translate("Time"),
				(int)(kubus->getTime()/1000) );
	statusbar->changeItem( tmp.data(), STATUS_TIME );

	tmp.sprintf("%d", kubus->getMoves() );
	counter->setText( tmp.data()  );
}

void KRubik::helpmenu()
{
	KApplication::getKApplication()->invokeHTMLHelp("","");
}

void KRubik::configkeys()
{
	KKeyDialog::configureKeys(accel);
}


void KRubik::initKeys()
{
	// here we create to shortcuts according to
	// the standard kde keybindings
	accel = new KAccel(this);

	accel->insertItem( i18n("About krubik"), "About", "CTRL+A" ) ;
	accel->connectItem( "About", this, SLOT(about()) ) ;
	
	accel->connectItem( KAccel::Help, this, SLOT( helpmenu()) ) ;
	accel->connectItem( KAccel::New, this, SLOT( newCube()) ) ;
	accel->connectItem( KAccel::Open, this, SLOT( loadCube()) ) ;
	accel->connectItem( KAccel::Save, this, SLOT( saveCube()) ) ;
	accel->connectItem( KAccel::Quit, this, SLOT( quitapp()) ) ;

	accel->readSettings(config) ;
}

void KRubik::saveCube()
{
	kubus->save();
}

void KRubik::newCube()
{
	kubus->newCube();
}

void KRubik::loadCube()
{
	kubus->load();
}

void KRubik::scrambleCube()
{
	kubus->scramble();
}

void KRubik::resetColors()
{
	kubus->resetColors();
}

void KRubik::setShading( int level )
{
	kubus->setShading( level );
}

int KRubik::getShading()
{
	return kubus->getShading();
}

void KRubik::initConfig()
{
	config = kapp->getConfig();
	
	config->setGroup("Options");

	if ( config->readNumEntry("Statusbar", 1)==1 )
	{
		showstatus=true;
	}
	else	showstatus=false;
	setShading( config->readNumEntry("Shading", Cube::NO_SHADE ) );

	config->setGroup("Colors");

	kubus->setColor( config->readColorEntry("Background",
			new QColor( backgroundColor()) ),
			BACKGROUND,
			false );
	kubus->setColor( config->readColorEntry("Edge",
			new QColor(black)),
			EDGE,
			false );

	kubus->setColor( config->readColorEntry("One",
			 new QColor(red)),
			 0,
			 false ) ;
	kubus->setColor( config->readColorEntry("Two",
			new QColor(blue)),
			1,
			false );
	kubus->setColor( config->readColorEntry("Three",
				new QColor(green)),
				2,
				false );
	kubus->setColor( config->readColorEntry("Four",
				new QColor(yellow)),
				3,
				false );
	kubus->setColor( config->readColorEntry("Five",
				new QColor(magenta)),
				4,
				false );
 	kubus->setColor( config->readColorEntry("Six",
				new QColor(white)),
				5,
				false );
}

void KRubik::saveConfig()
{
	// write the options
	config->setGroup("Options");

	if ( showstatus )	config->writeEntry("Statusbar", 1);
	else			config->writeEntry("Statusbar", 0);

	config->writeEntry("Shading", kubus->getShading() );

	// write the color-settings
	config->setGroup("Colors");

	config->writeEntry("Background",
			kubus->getBackgroundColor() );
	config->writeEntry("Edge", kubus->getEdgeColor() );

	config->writeEntry("One", kubus->getColor(0) );
	config->writeEntry("Two", kubus->getColor(1) );
	config->writeEntry("Three", kubus->getColor(2) );
	config->writeEntry("Four", kubus->getColor(3) );
	config->writeEntry("Five", kubus->getColor(4) );
	config->writeEntry("Six", kubus->getColor(5) );

	accel->writeSettings(config) ;

	config->sync();
}

void KRubik::cleanCube()
{
	kubus->clean();
}

KRubik:: KRubik ( QWidget *, const char* name )
	: KTopLevelWidget ( name )
{
	setCaption(CAPTION);
	setMinimumSize(350, 300);
	resize(500, 400);
	setMaximumSize(800, 600);

	box = new QGroupBox(klocale->translate("Cube"),this);
	kubus = new Cube(box);
	connect( kubus, SIGNAL(stateChange() ), this, SLOT(status()) );

	initConfig();

	moves = new QLabel("Moves :", this);
	counter = new QLabel("0", this);
	counter->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	counter->setAlignment( AlignCenter );
	
	clean = new QPushButton(klocale->translate("Clean cube"), this);
	connect (clean, SIGNAL(clicked()), this, SLOT(cleanCube()) );
	scramble = new QPushButton(klocale->translate("Scramble"), this);
	connect (scramble, SIGNAL(clicked()), this, SLOT(scrambleCube()) );

	createStatusBar();
	initKeys();
	createMenu();

	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(status()) );
	timer->start(500, false);
}

KRubik::~KRubik()
{
	delete kubus;
	delete moves;
	delete counter;
	delete clean;
	delete scramble;
	delete box;

	delete timer;
	delete statusbar;
	delete file;
	delete options;
	delete help;
	delete menu;
}

void KRubik::quitapp()
{
		saveConfig();
		krubik->quit();	
}

void KRubik::about()
{
	QString q;
	const char *msg = klocale->translate("A program written in"
			" C++ using Qt/Kde.\nBased on original code from"
			" Karl Hornell."
			"\n\nWritten by Pieter Eendebak <pte@ddsw.nl>");

	q.sprintf("%s %s (%s)\n\n%s", NAME, VERSION, DATE, msg);
	KMsgBox::message(this, CAPTION, q );
}


void KRubik::notImpl()
{
	KMsgBox::message(this, CAPTION,
	klocale->translate("Sorry, this function hasn't been implemented yet...")  );
}

void KRubik::configColors()
{
	int i=0;

	Kleur *k = new Kleur( this, 0 );
	
	k->background->setColor( kubus->getBackgroundColor() );
	k->edge->setColor( kubus->getEdgeColor() );	

	for(i=0;i<6;i++)
	{
		k->kleur[i]->setColor( kubus->getColor(i) );
	}

	k->shade->setChecked( getShading() != -1 );
	k->leveltoggle(true) ;
		
	if ( k->exec() )
	{
		// get and install colors...
	
		kubus->setColor( k->background->color(),
				BACKGROUND,
				false );
		kubus->setColor( k->edge->color(), EDGE, false);

		setShading( k->getShading() );

		for(i=0; i<6;i++)
		{
			kubus->setColor( k->kleur[i]->color(), i, false );
		}
	}
	delete k;

	kubus->repaint(false);

}


/* ##########################
 * #	Main                #
 * ##########################
 */

int main(int argc, char **argv)
{
	krubik = new KApplication(argc, argv, "krubik" );
	KRubik *top;
	top = new KRubik(0, "krubik");
	krubik->setMainWidget(top);
	top->show();
	return krubik->exec();
}

